﻿using System;
using Plugin;
using System.Windows.Forms;//MessageBox.Show( "" );

namespace ncvcommentmado
{
    public class Main : IPlugin
    {
        //--------------------
        // TODO:    Main フィールド
        //--------------------

        IPluginHost host = null;
        CommentForm commentForm = null;
        const string pluginName = "コメント窓";
        const string pluginVersion = "1.0";
        const string pluginDescription = "独立したコメント窓を表示する。";
        const string pluginSettingsDirName = "co1430045\\";
        const string pluginSettingsFileName = "ncvcommentmado";

        //--------------------
        // TODO:    Main IPlugin メソッド
        //--------------------

        public void Run()
        {
            host.BroadcastConnected += host_BroadcastConnected;
            host.BroadcastDisConnected += host_BroadcastDisConnected;

            initializeCommentForm();
            loadSettings();
        }

        public void AutoRun()
        {
            Run();
        }

        //--------------------
        // TODO:    Main IPlugin プロパティ
        //--------------------

        public bool IsAutoRun       { get { return true; } }
        public string Description   { get { return pluginDescription; } }
        public string Name          { get { return pluginName; } }
        public string Version       { get { return pluginVersion; } }
        public IPluginHost Host
        {
            get { return host; }
            set { host = value; }
        }

        //--------------------
        // TODO:    Main イベントハンドラ
        //--------------------

        void host_BroadcastConnected( object sender, EventArgs e )
        {
            commentForm.PostButtonEnabled = true;
        }

        void host_BroadcastDisConnected( object sender, EventArgs e )
        {
            commentForm.PostButtonEnabled = false;
        }

        void commentForm_FormClosed( 
            object sender, FormClosedEventArgs e )
        {
            saveSettings();
            uninitializeCommentForm();
        }

        //--------------------
        // TODO:    Main デリゲート
        //--------------------

        bool commentForm_PostCommand( 
            bool isOwner, bool isAnonymous, string comment )
        {
            if (isOwner)
            {
                return host.SendOwnerComment( 
                    isAnonymous, "", "", "", comment, "", "" );
            }
            else
            {
                return host.SendComment( isAnonymous, "", "", "", comment );
            }
        }

        //--------------------
        // TODO:    Main サブルーチン
        //--------------------

        void initializeCommentForm()
        {
            commentForm = new CommentForm();
            commentForm.Text = pluginName;

            commentForm.PostCommand += commentForm_PostCommand;
            commentForm.FormClosed += commentForm_FormClosed;

            commentForm.Show( host.MainForm ); //NCV本体とアクティブ状態を連動させない
        }

        void loadSettings()
        {
            /* ウィンドウの位置とサイズを設定ファイルから読みだして反映 */
            string path = host.DirectoryPathSettingFile 
                + pluginSettingsDirName
                + pluginSettingsFileName;
            string[] settings;

            try
            {
                settings = System.IO.File.ReadAllLines( path );
            }
            catch( System.IO.FileNotFoundException e )
            {
                return;
            }

            foreach ( string line in settings )
            {
                int NAME = 0;
                int VALUE = 1;
                string[] setting = line.Split( ':' );
                switch( setting[NAME] )
                {
                    case "xpos":
                        commentForm.Left = Convert.ToInt32( setting[VALUE] );
                        break;
                    case "ypos":
                        commentForm.Top = Convert.ToInt32( setting[VALUE] );
                        break;
                    case "width":
                        commentForm.Width = Convert.ToInt32( setting[VALUE] );
                        break;
                    case "height":
                        commentForm.Height = Convert.ToInt32( setting[VALUE] );
                        break;
                }
            }
        }

        void saveSettings()
        {
            string settings = "xpos:" + commentForm.Location.X
                + "\nypos:" + commentForm.Location.Y
                + "\nwidth:" + commentForm.Width
                + "\nheight:" + commentForm.Height;

            System.IO.Directory.CreateDirectory( 
                host.DirectoryPathSettingFile + pluginSettingsDirName );
            System.IO.File.WriteAllText( 
                host.DirectoryPathSettingFile + pluginSettingsDirName 
                + pluginSettingsFileName, 
                settings );
        }

        void uninitializeCommentForm()
        {
            host.BroadcastConnected -= host_BroadcastConnected;
            host.BroadcastDisConnected -= host_BroadcastDisConnected;
            commentForm.PostCommand -= commentForm_PostCommand;
            commentForm.FormClosed -= commentForm_FormClosed;
        }
    }
}
